@@ -31,7 +31,6 @@ const CheckIn = imports.checkIn;
 const ContactPlace = imports.contactPlace;
 const Format = imports.format;
 const Geoclue = imports.geoclue;
-const GeocodeService = imports.geocodeService;
 const MainWindow = imports.mainWindow;
 const Maps = imports.gi.GnomeMaps;
 const OSMEdit = imports.osmEdit;
@@ -48,7 +47,6 @@ var settings = null;
 var placeStore = null;
 var routingDelegator = null;
 var geoclue = null;
-var geocodeService = null;
 var networkMonitor = null;
 var checkInManager = null;
 var contactStore = null;
@@ -159,19 +157,29 @@ var Application = GObject.registerClass({
     }
 
     _addContacts() {
-        contactStore.get_contacts().forEach((contact) => {
-            contact.geocode(function() {
+        let contacts = contactStore.get_contacts();
+
+        this._addContactsRecursive(contacts, 0);
+    }
+
+    _addContactsRecursive(contacts, index) {
+        if (index < contacts.length) {
+            let contact = contacts[index];
+
+            contact.geocode(() => {
                 contact.get_places().forEach((p) => {
                     if (!p.location)
-                        return;
+                        return
 
                     Utils.debug('Adding contact address: ' + p.name);
                     let place = new ContactPlace.ContactPlace({ place: p,
                                                                 contact: contact });
                     placeStore.addPlace(place, PlaceStore.PlaceType.CONTACT);
                 });
+
+                this._addContactsRecursive(contacts, index + 1);
             });
-        });
+        }
     }
 
     _initPlaceStore() {
@@ -230,7 +238,6 @@ var Application = GObject.registerClass({
         routeQuery       = new RouteQuery.RouteQuery();
         routingDelegator = new RoutingDelegator.RoutingDelegator({ query: routeQuery });
         geoclue          = new Geoclue.Geoclue();
-        geocodeService   = new GeocodeService.GeocodeService();
         networkMonitor   = Gio.NetworkMonitor.get_default();
         networkMonitor.connect('network-changed',
                                this._checkNetwork.bind(this));
