@@ -27,12 +27,19 @@ const Gio = imports.gi.Gio;
 const Gtk = imports.gi.Gtk;
 
 const Application = imports.application;
+const GeocodeFactory = imports.geocode;
 const Location = imports.location;
 const Place = imports.place;
 const PlaceStore = imports.placeStore;
 const PlacePopover = imports.placePopover;
 const Utils = imports.utils;
 
+// minimum number of characters to start completion
+const MIN_CHARS_COMPLETION = 3;
+
+// pattern matching CJK ideographic characters
+const IDEOGRAPH_PATTERN = /[\u3300-\u9fff]/
+
 var PlaceEntry = GObject.registerClass({
     Properties: {
         'place': GObject.ParamSpec.object('place',
@@ -53,11 +60,13 @@ var PlaceEntry = GObject.registerClass({
 
         if (p) {
             if (p.name) {
-                this.text = p.name;
+                this._placeText = p.name;
             } else
-                this.text = p.location.latitude + ', ' + p.location.longitude;
+                this._placeText = p.location.latitude + ', ' + p.location.longitude;
         } else
-            this.text = '';
+            this._placeText = '';
+
+        this.text = this._placeText;
 
         this._place = p;
         this.notify('place');
@@ -84,9 +93,6 @@ var PlaceEntry = GObject.registerClass({
         let maxChars = props.maxChars;
         delete props.maxChars;
 
-        let parseOnFocusOut = props.parseOnFocusOut;
-        delete props.parseOnFocusOut;
-
         this._matchRoute = props.matchRoute || false;
         delete props.matchRoute;
 
@@ -97,30 +103,48 @@ var PlaceEntry = GObject.registerClass({
 
         this._popover = this._createPopover(numVisible, maxChars);
 
-        this.connect('activate', this._onActivate.bind(this));
-        this.connect('search-changed', () => {
-            if (this._cancellable)
-                this._cancellable.cancel();
+        this.connect('search-changed', this._onSearchChanged.bind(this));
 
-            this._refreshFilter();
+        this._cache = {};
 
-            if (this.text.length === 0) {
-                this._popover.hide();
-                this.place = null;
-                return;
+        // clear cache when view moves, as result are location-dependent
+        this._mapView.view.connect('notify::latitude', () => this._cache = {});
+    }
+
+    _onSearchChanged() {
+        if (this._parse())
+            return;
+
+        // wait for an ongoing search
+        if (this._cancellable)
+            return;
+
+        /* start search if more than the threashold number of characters have
+         * been entered, or if the first character is in the ideographic CJK
+         * block, as for these, shorter strings could be meaningful
+         */
+        if ((this.text.length >= MIN_CHARS_COMPLETION ||
+             (this.text.length > 0 && this.text[0].match(IDEOGRAPH_PATTERN))) &&
+            this.text !== this._placeText) {
+            let cachedResults = this._cache[this.text];
+
+            if (cachedResults) {
+                this._updateResults(cachedResults);
+            } else {
+                // if no previous search has been performed, show spinner
+                if (!this._previousSearch ||
+                    this._previousSearch.length < MIN_CHARS_COMPLETION ||
+                    this._placeText) {
+                    this._popover.showSpinner();
+                }
+                this._placeText = '';
+                this._doSearch();
             }
-
-            if (this._filter.iter_n_children(null) > 0)
-                this._popover.showCompletion();
-            else
-                this._popover.hide();
-        });
-
-        if (parseOnFocusOut) {
-            this.connect('focus-out-event', () => {
-                this._parse();
-                return false;
-            });
+        } else {
+            this._popover.hide();
+            if (this.text.length === 0)
+                this.place = null;
+            this._previousSearch = null;
         }
     }
 
@@ -151,17 +175,13 @@ var PlaceEntry = GObject.registerClass({
         return popover;
     }
 
-    _refreshFilter() {
-        /* Filter model based on input text */
-        this._filter.refilter();
-        this._popover.updateCompletion(this._filter, this.text);
-    }
-
     _completionVisibleFunc(model, iter) {
         let place = model.get_value(iter, PlaceStore.Columns.PLACE);
         let type = model.get_value(iter, PlaceStore.Columns.TYPE);
 
-        if (!this._matchRoute && type === PlaceStore.PlaceType.RECENT_ROUTE)
+        if (type !== PlaceStore.PlaceType.CONTACT &&
+            type !== PlaceStore.PlaceType.RECENT_ROUTE ||
+            (!this._matchRoute && type === PlaceStore.PlaceType.RECENT_ROUTE))
             return false;
 
         if (place !== null)
@@ -171,10 +191,7 @@ var PlaceEntry = GObject.registerClass({
     }
 
     _parse() {
-        if (this.text.length === 0) {
-            this.place = null;
-            return true;
-        }
+        let parsed = false;
 
         if (this.text.startsWith('geo:')) {
             let location = new Geocode.Location();
@@ -187,34 +204,77 @@ var PlaceEntry = GObject.registerClass({
                 Utils.showDialog(msg, Gtk.MessageType.ERROR, this.get_toplevel());
             }
 
-            return true;
+            parsed = true;
         }
 
         let parsedLocation = Place.Place.parseCoordinates(this.text);
         if (parsedLocation) {
             this.place = new Place.Place({ location: parsedLocation });
-            return true;
+            parsed = true;
         }
 
-        return false;
+        if (parsed && this._cancellable)
+            this._cancellable.cancel();
+
+        return parsed;
     }
 
-    _onActivate() {
-        if (this._parse())
-            return;
-
-        let bbox = this._mapView.view.get_bounding_box();
-
-        this._popover.showSpinner();
+    _doSearch() {
+        if (this._cancellable)
+            this._cancellable.cancel();
         this._cancellable = new Gio.Cancellable();
-        Application.geocodeService.search(this.text, bbox, this._cancellable, (places) => {
-            if (!places) {
+        this._previousSearch = this.text;
+        GeocodeFactory.getGeocoder().search(this.text,
+                                            this._mapView.view.latitude,
+                                            this._mapView.view.longitude,
+                                            this._cancellable,
+                                            (places, error) => {
+            this._cancellable = null;
+            this._updateResults(places);
+
+            // cache results for later
+            this._cache[this.text] = places;
+
+            // if search input has been updated, trigger a refresh
+            if (this.text !== this._previousSearch)
+                this._onSearchChanged();
+        });
+    }
+
+    _updateResults(places) {
+        if (!places) {
                 this.place = null;
                 this._popover.showNoResult();
                 return;
-            }
-            this._popover.updateResult(places, this.text);
-            this._popover.showResult();
+        }
+
+        let completedPlaces = [];
+
+
+        this._filter.refilter();
+        this._filter.foreach((model, path, iter) => {
+            let place = model.get_value(iter, PlaceStore.Columns.PLACE);
+            let type = model.get_value(iter, PlaceStore.Columns.TYPE);
+
+            completedPlaces.push({ place: place, type: type });
         });
+
+        let placeStore = Application.placeStore;
+
+        places.forEach((place) => {
+            let type;
+
+            if (placeStore.exists(place, PlaceStore.PlaceType.RECENT))
+                type = PlaceStore.PlaceType.RECENT;
+            else if (placeStore.exists(place, PlaceStore.PlaceType.FAVORITE))
+                type = PlaceStore.PlaceType.FAVORITE;
+            else
+                type = PlaceStore.PlaceType.ANY;
+
+            completedPlaces.push({ place: place, type: type });
+        });
+
+        this._popover.updateResult(completedPlaces, this.text);
+        this._popover.showResult();
     }
 });
